@@ -41,6 +41,7 @@
 #include "gal/widgets/e-canvas-utils.h"
 #include "gal/util/e-util.h"
 #include <string.h>
+#include <stdlib.h>
 
 #define PARENT_OBJECT_TYPE gnome_canvas_item_get_type ()
 
@@ -104,6 +105,8 @@ static void e_table_item_focus (ETableItem *eti, int col, int row, GdkModifierTy
 static void eti_cursor_change (ESelectionModel *selection, int row, int col, ETableItem *eti);
 static void eti_cursor_activated (ESelectionModel *selection, int row, int col, ETableItem *eti);
 static void eti_selection_change (ESelectionModel *selection, ETableItem *eti);
+static void eti_selection_row_change (ESelectionModel *selection, int row, ETableItem *eti);
+static void e_table_item_redraw_row (ETableItem *eti, int row);
 
 #define ETI_SINGLE_ROW_HEIGHT(eti) ((eti)->uniform_row_height_cache != -1 ? (eti)->uniform_row_height_cache : eti_row_height((eti), -1))
 #define ETI_MULTIPLE_ROW_HEIGHT(eti,row) ((eti)->height_cache && (eti)->height_cache[(row)] != -1 ? (eti)->height_cache[(row)] : eti_row_height((eti),(row)))
@@ -498,6 +501,8 @@ eti_remove_selection_model (ETableItem *eti)
 
 	gtk_signal_disconnect (GTK_OBJECT (eti->selection),
 			       eti->selection_change_id);
+	gtk_signal_disconnect (GTK_OBJECT (eti->selection),
+			       eti->selection_row_change_id);
 	gtk_signal_disconnect (GTK_OBJECT (eti->selection),
 			       eti->cursor_change_id);
 	gtk_signal_disconnect (GTK_OBJECT (eti->selection),
@@ -505,6 +510,7 @@ eti_remove_selection_model (ETableItem *eti)
 	gtk_object_unref (GTK_OBJECT (eti->selection));
 
 	eti->selection_change_id = 0;
+	eti->selection_row_change_id = 0;
 	eti->cursor_activated_id = 0;
 	eti->selection = NULL;
 }
@@ -604,19 +610,21 @@ height_cache_idle(ETableItem *eti)
 static void
 free_height_cache (ETableItem *eti)
 {
-	if (eti->height_cache)
-		g_free (eti->height_cache);
-	eti->height_cache = NULL;
-	eti->height_cache_idle_count = 0;
-	eti->uniform_row_height_cache = -1;
+	if (GNOME_CANVAS_ITEM_REALIZED & GTK_OBJECT_FLAGS (eti)) {
+		if (eti->height_cache)
+			g_free (eti->height_cache);
+		eti->height_cache = NULL;
+		eti->height_cache_idle_count = 0;
+		eti->uniform_row_height_cache = -1;
 
-	if (eti->uniform_row_height && eti->height_cache_idle_id != 0) {
-		g_source_remove(eti->height_cache_idle_id);
-		eti->height_cache_idle_id = 0;
+		if (eti->uniform_row_height && eti->height_cache_idle_id != 0) {
+			g_source_remove(eti->height_cache_idle_id);
+			eti->height_cache_idle_id = 0;
+		}
+
+		if ((!eti->uniform_row_height) && eti->height_cache_idle_id == 0)
+			eti->height_cache_idle_id = g_idle_add_full(G_PRIORITY_LOW, (GSourceFunc) height_cache_idle, eti, NULL);
 	}
-
-	if ((!eti->uniform_row_height) && eti->height_cache_idle_id == 0)
-		eti->height_cache_idle_id = g_idle_add_full(G_PRIORITY_LOW, (GSourceFunc) height_cache_idle, eti, NULL);
 }
 
 static void
@@ -973,13 +981,15 @@ static void
 eti_freeze (ETableItem *eti)
 {
 	eti->frozen_count ++;
+	d(g_print ("%s: %d\n", __FUNCTION__, eti->frozen_count));
 }
 
 static void
 eti_unfreeze (ETableItem *eti)
 {
-	g_return_if_fail (eti->frozen_count != 0);
+	g_return_if_fail (eti->frozen_count > 0);
 	eti->frozen_count --;
+	d(g_print ("%s: %d\n", __FUNCTION__, eti->frozen_count));
 	if (eti->frozen_count == 0 && eti->queue_show_cursor) {
 		eti_show_cursor (eti, 0);
 		eti_check_cursor_bounds (eti);
@@ -1050,7 +1060,7 @@ eti_table_model_row_changed (ETableModel *table_model, int row, ETableItem *eti)
 
 	eti_unfreeze (eti);
 
-	eti_request_region_redraw (eti, 0, row, eti->cols - 1, row, 0);
+	e_table_item_redraw_row (eti, row);
 }
 
 static void
@@ -1068,7 +1078,7 @@ eti_table_model_cell_changed (ETableModel *table_model, int col, int row, ETable
 
 	eti_unfreeze (eti);
 
-	eti_request_region_redraw (eti, 0, row, eti->cols - 1, row, 0);
+	e_table_item_redraw_row (eti, row);
 }
 
 static void
@@ -1164,6 +1174,13 @@ e_table_item_redraw_range (ETableItem *eti,
 	eti_request_region_redraw(eti, start_col, start_row, end_col, end_row, border);
 }
 
+static void
+e_table_item_redraw_row (ETableItem *eti,
+			 int row)
+{
+	e_table_item_redraw_range (eti, 0, row, eti->cols - 1, row);
+}
+
 static void
 eti_add_table_model (ETableItem *eti, ETableModel *table_model)
 {
@@ -1228,6 +1245,10 @@ eti_add_selection_model (ETableItem *eti, ESelectionModel *selection)
 		GTK_OBJECT (selection), "selection_changed",
 		GTK_SIGNAL_FUNC (eti_selection_change), eti);
 
+	eti->selection_row_change_id = gtk_signal_connect (
+		GTK_OBJECT (selection), "selection_row_changed",
+		GTK_SIGNAL_FUNC (eti_selection_row_change), eti);
+
 	eti->cursor_change_id = gtk_signal_connect (
 		GTK_OBJECT (selection), "cursor_changed",
 		GTK_SIGNAL_FUNC (eti_cursor_change), eti);
@@ -1494,10 +1515,13 @@ eti_init (GnomeCanvasItem *item)
 	eti->cursor_mode               = E_CURSOR_SIMPLE;
 
 	eti->selection_change_id       = 0;
+	eti->selection_row_change_id   = 0;
 	eti->cursor_change_id          = 0;
 	eti->cursor_activated_id       = 0;
 	eti->selection                 = NULL;
 
+	eti->old_cursor_row            = -1;
+
 	eti->needs_redraw              = 0;
 	eti->needs_compute_height      = 0;
 
@@ -1617,6 +1641,16 @@ eti_unrealize (GnomeCanvasItem *item)
 	if (eti_editing (eti))
 		e_table_item_leave_edit_(eti);
 
+	if (eti->height_cache_idle_id) {
+		g_source_remove(eti->height_cache_idle_id);
+		eti->height_cache_idle_id = 0;
+	}
+
+	if (eti->height_cache)
+		g_free (eti->height_cache);
+	eti->height_cache = NULL;
+	eti->height_cache_idle_count = 0;
+
 	gdk_gc_unref (eti->fill_gc);
 	eti->fill_gc = NULL;
 	gdk_gc_unref (eti->grid_gc);
@@ -2663,7 +2697,7 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 	default:
 		return_val = FALSE;
 	}
-	d(g_print("%s: returning: %s\n", __FUNCTION__, return_val?"true":"false"));
+	/* d(g_print("%s: returning: %s\n", __FUNCTION__, return_val?"true":"false"));*/
 
 	return return_val;
 }
@@ -2880,6 +2914,7 @@ eti_cursor_change (ESelectionModel *selection, int row, int col, ETableItem *eti
 	
 	if (view_row == -1 || view_col == -1) {
 		e_table_item_leave_edit_(eti);
+		eti->old_cursor_row = -1;
 		return;
 	}
 
@@ -2896,8 +2931,14 @@ eti_cursor_change (ESelectionModel *selection, int row, int col, ETableItem *eti
 		e_table_item_leave_edit_(eti);
 	gtk_signal_emit (GTK_OBJECT (eti), eti_signals [CURSOR_CHANGE],
 			 view_row);
-	eti->needs_redraw = TRUE;
-	gnome_canvas_item_request_update(GNOME_CANVAS_ITEM(eti));
+	if (eti->old_cursor_row != -1) {
+		e_table_item_redraw_row (eti, eti->old_cursor_row);
+		e_table_item_redraw_row (eti, view_row);
+	} else {
+		eti->needs_redraw = TRUE;
+		gnome_canvas_item_request_update(GNOME_CANVAS_ITEM(eti));
+	}
+	eti->old_cursor_row = view_row;
 }
 
 static void
@@ -2939,6 +2980,17 @@ eti_selection_change (ESelectionModel *selection, ETableItem *eti)
 	gnome_canvas_item_request_update(GNOME_CANVAS_ITEM(eti));
 }
 
+static void
+eti_selection_row_change (ESelectionModel *selection, int row, ETableItem *eti)
+{
+	if (!(GTK_OBJECT_FLAGS(eti) & GNOME_CANVAS_ITEM_REALIZED))
+		return;
+
+	if (!eti->needs_redraw) {
+		e_table_item_redraw_row (eti, model_to_view_row(eti, row));
+	}
+}
+
 
 /** 
  * e_table_item_enter_edit
