@@ -1,25 +1,24 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-
 /* 
- * Author : 
- *  Damon Chaplin <damon@helixcode.com>
+ * e-shortcut-bar.c
+ * Copyright 1999, 2000, 2001, Ximian, Inc.
  *
- * Copyright 1999, Helix Code, Inc.
+ * Authors:
+ *   Damon Chaplin <damon@ximian.com>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 
 /*
@@ -196,7 +195,7 @@ e_shortcut_bar_class_init (EShortcutBarClass *class)
 	e_shortcut_bar_signals[ITEM_SELECTED] =
 		gtk_signal_new ("item_selected",
 				GTK_RUN_LAST | GTK_RUN_ACTION,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (EShortcutBarClass,
 						   item_selected),
 				gtk_marshal_NONE__POINTER_INT_INT,
@@ -206,7 +205,7 @@ e_shortcut_bar_class_init (EShortcutBarClass *class)
 	e_shortcut_bar_signals[SHORTCUT_DROPPED] =
 		gtk_signal_new ("shortcut_dropped",
 				GTK_RUN_LAST | GTK_RUN_ACTION,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (EShortcutBarClass, shortcut_dropped),
 				e_shortcut_bar_marshal_NONE__INT_INT_STRING_STRING,
 				GTK_TYPE_NONE, 4,
@@ -218,14 +217,14 @@ e_shortcut_bar_class_init (EShortcutBarClass *class)
 	e_shortcut_bar_signals[SHORTCUT_DRAGGED] =
 		gtk_signal_new ("shortcut_dragged",
 				GTK_RUN_LAST | GTK_RUN_ACTION,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (EShortcutBarClass, shortcut_dragged),
 				gtk_marshal_NONE__INT_INT,
 				GTK_TYPE_NONE, 2,
 				GTK_TYPE_INT,
 				GTK_TYPE_INT);
 
-	gtk_object_class_add_signals (object_class, e_shortcut_bar_signals,
+	E_OBJECT_CLASS_ADD_SIGNALS (object_class, e_shortcut_bar_signals,
 				      LAST_SIGNAL);
 
 	/* Method override */
@@ -509,6 +508,7 @@ e_shortcut_bar_add_group	(EShortcutBar	*shortcut_bar,
 	gtk_object_set(GTK_OBJECT(label),
 		       "draw_background", FALSE,
 		       "draw_borders", FALSE,
+		       "draw_button", TRUE,
 		       "editable", FALSE,
 		       "text", group_name,
 		       "use_ellipsis", TRUE,
