@@ -1,5 +1,26 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* This code is GPL. */
+/* 
+ * test-table-1.c
+ * Copyright 2000, 2001, Ximian, Inc.
+ *
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
+ */
+
 #include <stdio.h>
 #include <string.h>
 
@@ -59,18 +80,18 @@ char *headers [COLS] = {
 };
 #endif
 
-#define SPEC "<ETableSpecification cursor-mode=\"line\" selection-mode=\"browse\" draw-focus=\"true\">                    	       \
-  <ETableColumn model_col=\"0\" _title=\"Email\"   expansion=\"1.0\" minimum_width=\"20\" resizable=\"true\" cell=\"string\" compare=\"string\"/> \
-  <ETableColumn model_col=\"1\" _title=\"Full Name\" expansion=\"1.0\" minimum_width=\"20\" resizable=\"true\" cell=\"string\"      compare=\"string\"/> \
-  <ETableColumn model_col=\"2\" _title=\"Address\"     expansion=\"1.0\" minimum_width=\"20\" resizable=\"true\" cell=\"string\"      compare=\"string\"/> \
-  <ETableColumn model_col=\"3\" _title=\"Phone\"      expansion=\"1.0\" minimum_width=\"20\" resizable=\"true\" cell=\"string\"      compare=\"string\"/> \
-        <ETableState> \
-	        <column source=\"0\"/> \
-	        <column source=\"1\"/> \
-	        <column source=\"2\"/> \
-	        <column source=\"3\"/> \
-	        <grouping> <leaf column=\"1\" ascending=\"true\"/> </grouping>    \
-        </ETableState> \
+#define SPEC "<ETableSpecification cursor-mode=\"line\" selection-mode=\"browse\" draw-focus=\"true\" allow-grouping=\"false\">				\
+  <ETableColumn model_col=\"0\" _title=\"Email\"   expansion=\"1.0\" minimum_width=\"20\" resizable=\"true\" cell=\"string\"     compare=\"string\"/>	\
+  <ETableColumn model_col=\"1\" _title=\"Full Name\" expansion=\"1.0\" minimum_width=\"20\" resizable=\"true\" cell=\"string\"   compare=\"string\"/>	\
+  <ETableColumn model_col=\"2\" _title=\"Address\"     expansion=\"1.0\" minimum_width=\"20\" resizable=\"true\" cell=\"string\" compare=\"string\"/>	\
+  <ETableColumn model_col=\"3\" _title=\"Phone\"      expansion=\"1.0\" minimum_width=\"20\" resizable=\"true\" cell=\"string\"  compare=\"string\"/>	\
+        <ETableState>																	\
+	        <column source=\"0\"/>															\
+	        <column source=\"1\"/>															\
+	        <column source=\"2\"/>															\
+	        <column source=\"3\"/>															\
+	        <grouping> <leaf column=\"1\" ascending=\"true\"/> </grouping>										\
+        </ETableState>																	\
 </ETableSpecification>"
 
 /*
